# Generated by roxygen2: do not edit by hand

S3method(as.data.frame,mfrm_fit)
S3method(as.data.frame,mfrm_quadrature_sensitivity)
S3method(as_flextable,apa_table)
S3method(as_ggplot,default)
S3method(as_ggplot,mfrm_design_evaluation)
S3method(as_ggplot,mfrm_design_evaluation_plot_data)
S3method(as_ggplot,mfrm_plot_data)
S3method(as_ggplot,mfrm_signal_detection)
S3method(as_ggplot,mfrm_signal_detection_plot_data)
S3method(as_kable,apa_table)
S3method(plot,apa_table)
S3method(plot,mfrm_anchor_review)
S3method(plot,mfrm_bundle)
S3method(plot,mfrm_d_study)
S3method(plot,mfrm_data_description)
S3method(plot,mfrm_design_evaluation)
S3method(plot,mfrm_diagnostic_screening)
S3method(plot,mfrm_equating_chain)
S3method(plot,mfrm_facet_dashboard)
S3method(plot,mfrm_facet_equivalence)
S3method(plot,mfrm_facet_nesting)
S3method(plot,mfrm_facet_sample_review)
S3method(plot,mfrm_facets_run)
S3method(plot,mfrm_fit)
S3method(plot,mfrm_hierarchical_structure)
S3method(plot,mfrm_recovery_assessment)
S3method(plot,mfrm_recovery_simulation)
S3method(plot,mfrm_response_time_review)
S3method(plot,mfrm_results)
S3method(plot,mfrm_signal_detection)
S3method(plot,mfrm_structural_design_review)
S3method(plot,mfrm_summary_table_bundle)
S3method(print,apa_table)
S3method(print,mfrm_anchor_drift)
S3method(print,mfrm_anchor_review)
S3method(print,mfrm_anchored_fit)
S3method(print,mfrm_apa_outputs)
S3method(print,mfrm_apa_text)
S3method(print,mfrm_bias)
S3method(print,mfrm_bundle)
S3method(print,mfrm_comparison)
S3method(print,mfrm_d_study)
S3method(print,mfrm_data_description)
S3method(print,mfrm_design_evaluation)
S3method(print,mfrm_dff)
S3method(print,mfrm_diagnostics)
S3method(print,mfrm_dif)
S3method(print,mfrm_dif_interaction)
S3method(print,mfrm_dif_report)
S3method(print,mfrm_equating_chain)
S3method(print,mfrm_facet_dashboard)
S3method(print,mfrm_facet_design_effect)
S3method(print,mfrm_facet_icc)
S3method(print,mfrm_facet_nesting)
S3method(print,mfrm_facet_sample_review)
S3method(print,mfrm_facets_run)
S3method(print,mfrm_fit)
S3method(print,mfrm_generalizability)
S3method(print,mfrm_hierarchical_structure)
S3method(print,mfrm_imported_fit)
S3method(print,mfrm_linking_review)
S3method(print,mfrm_misfit_casebook)
S3method(print,mfrm_model_choice_review)
S3method(print,mfrm_network_review)
S3method(print,mfrm_peer_review_design_review)
S3method(print,mfrm_plausible_values)
S3method(print,mfrm_plot_bundle)
S3method(print,mfrm_plot_data)
S3method(print,mfrm_population_prediction)
S3method(print,mfrm_q3)
S3method(print,mfrm_qc_pipeline)
S3method(print,mfrm_quadrature_sensitivity)
S3method(print,mfrm_recovery_assessment)
S3method(print,mfrm_recovery_simulation)
S3method(print,mfrm_report)
S3method(print,mfrm_report_html)
S3method(print,mfrm_reporting_checklist)
S3method(print,mfrm_resamples)
S3method(print,mfrm_resampling_spec)
S3method(print,mfrm_response_time_review)
S3method(print,mfrm_results)
S3method(print,mfrm_results_export)
S3method(print,mfrm_results_html)
S3method(print,mfrm_signal_detection)
S3method(print,mfrm_sim_spec)
S3method(print,mfrm_structural_design_review)
S3method(print,mfrm_summary_table_bundle)
S3method(print,mfrm_threshold_profiles)
S3method(print,mfrm_unit_prediction)
S3method(print,mfrm_weighting_review)
S3method(print,mfrmr_gpcm_capabilities)
S3method(print,summary.apa_table)
S3method(print,summary.mfrm_anchor_drift)
S3method(print,summary.mfrm_anchor_review)
S3method(print,summary.mfrm_anchored_fit)
S3method(print,summary.mfrm_apa_outputs)
S3method(print,summary.mfrm_bias)
S3method(print,summary.mfrm_bundle)
S3method(print,summary.mfrm_comparison)
S3method(print,summary.mfrm_data_description)
S3method(print,summary.mfrm_design_evaluation)
S3method(print,summary.mfrm_dff)
S3method(print,summary.mfrm_diagnostic_screening)
S3method(print,summary.mfrm_diagnostics)
S3method(print,summary.mfrm_dif)
S3method(print,summary.mfrm_dif_interaction)
S3method(print,summary.mfrm_dif_report)
S3method(print,summary.mfrm_equating_chain)
S3method(print,summary.mfrm_facet_dashboard)
S3method(print,summary.mfrm_facets_run)
S3method(print,summary.mfrm_fit)
S3method(print,summary.mfrm_linking_review)
S3method(print,summary.mfrm_misfit_casebook)
S3method(print,summary.mfrm_model_choice_review)
S3method(print,summary.mfrm_network_review)
S3method(print,summary.mfrm_peer_review_design_review)
S3method(print,summary.mfrm_person_fit_indices)
S3method(print,summary.mfrm_plausible_values)
S3method(print,summary.mfrm_population_prediction)
S3method(print,summary.mfrm_qc_pipeline)
S3method(print,summary.mfrm_quadrature_sensitivity)
S3method(print,summary.mfrm_recovery_assessment)
S3method(print,summary.mfrm_recovery_simulation)
S3method(print,summary.mfrm_report)
S3method(print,summary.mfrm_reporting_checklist)
S3method(print,summary.mfrm_resamples)
S3method(print,summary.mfrm_resampling_spec)
S3method(print,summary.mfrm_response_time_review)
S3method(print,summary.mfrm_results)
S3method(print,summary.mfrm_signal_detection)
S3method(print,summary.mfrm_structural_design_review)
S3method(print,summary.mfrm_summary_table_bundle)
S3method(print,summary.mfrm_threshold_profiles)
S3method(print,summary.mfrm_unit_prediction)
S3method(print,summary.mfrm_weighting_review)
S3method(summary,apa_table)
S3method(summary,mfrm_anchor_drift)
S3method(summary,mfrm_anchor_review)
S3method(summary,mfrm_anchored_fit)
S3method(summary,mfrm_apa_outputs)
S3method(summary,mfrm_bias)
S3method(summary,mfrm_bundle)
S3method(summary,mfrm_comparison)
S3method(summary,mfrm_data_description)
S3method(summary,mfrm_design_evaluation)
S3method(summary,mfrm_dff)
S3method(summary,mfrm_diagnostic_screening)
S3method(summary,mfrm_diagnostics)
S3method(summary,mfrm_dif)
S3method(summary,mfrm_dif_interaction)
S3method(summary,mfrm_dif_report)
S3method(summary,mfrm_equating_chain)
S3method(summary,mfrm_facet_dashboard)
S3method(summary,mfrm_facet_design_effect)
S3method(summary,mfrm_facet_icc)
S3method(summary,mfrm_facet_nesting)
S3method(summary,mfrm_facet_sample_review)
S3method(summary,mfrm_facets_run)
S3method(summary,mfrm_fit)
S3method(summary,mfrm_hierarchical_structure)
S3method(summary,mfrm_linking_review)
S3method(summary,mfrm_misfit_casebook)
S3method(summary,mfrm_model_choice_review)
S3method(summary,mfrm_network_review)
S3method(summary,mfrm_peer_review_design_review)
S3method(summary,mfrm_person_fit_indices)
S3method(summary,mfrm_plausible_values)
S3method(summary,mfrm_population_prediction)
S3method(summary,mfrm_qc_pipeline)
S3method(summary,mfrm_quadrature_sensitivity)
S3method(summary,mfrm_recovery_assessment)
S3method(summary,mfrm_recovery_simulation)
S3method(summary,mfrm_report)
S3method(summary,mfrm_reporting_checklist)
S3method(summary,mfrm_resamples)
S3method(summary,mfrm_resampling_spec)
S3method(summary,mfrm_response_time_review)
S3method(summary,mfrm_results)
S3method(summary,mfrm_signal_detection)
S3method(summary,mfrm_structural_design_review)
S3method(summary,mfrm_summary_table_bundle)
S3method(summary,mfrm_threshold_profiles)
S3method(summary,mfrm_unit_prediction)
S3method(summary,mfrm_weighting_review)
export(analyze_dff)
export(analyze_dif)
export(analyze_facet_equivalence)
export(analyze_hierarchical_structure)
export(analyze_residual_pca)
export(anchor_review)
export(anchor_to_baseline)
export(apa_table)
export(apply_empirical_bayes_shrinkage)
export(as_flextable)
export(as_ggplot)
export(as_kable)
export(assess_mfrm_recovery)
export(bias_count_table)
export(bias_interaction_report)
export(bias_iteration_report)
export(bias_pairwise_report)
export(build_apa_outputs)
export(build_conquest_overlap_bundle)
export(build_equating_chain)
export(build_fixed_reports)
export(build_linking_review)
export(build_mfrm_manifest)
export(build_mfrm_network_review)
export(build_mfrm_replay_script)
export(build_mfrm_resampling_spec)
export(build_mfrm_sim_spec)
export(build_misfit_casebook)
export(build_model_choice_review)
export(build_peer_review_design_review)
export(build_peer_review_sim_spec)
export(build_summary_table_bundle)
export(build_visual_summaries)
export(build_weighting_review)
export(category_curves_report)
export(category_structure_report)
export(compare_mfrm)
export(compatibility_alias_table)
export(compute_facet_design_effect)
export(compute_facet_icc)
export(compute_information)
export(compute_person_fit_indices)
export(data_quality_report)
export(describe_mfrm_data)
export(detect_anchor_drift)
export(detect_facet_nesting)
export(diagnose_mfrm)
export(dif_interaction_table)
export(dif_report)
export(displacement_table)
export(draw_mfrm_resamples)
export(estimate_all_bias)
export(estimate_bias)
export(estimation_iteration_report)
export(evaluate_mfrm_design)
export(evaluate_mfrm_diagnostic_screening)
export(evaluate_mfrm_recovery)
export(evaluate_mfrm_signal_detection)
export(export_mfrm)
export(export_mfrm_bundle)
export(export_mfrm_results)
export(export_summary_appendix)
export(extract_mfrm_sim_spec)
export(facet_quality_dashboard)
export(facet_small_sample_review)
export(facet_statistics_report)
export(facets_chisq_table)
export(facets_feature_coverage)
export(facets_fit_df_guide)
export(facets_fit_review)
export(facets_output_contract_review)
export(facets_output_file_bundle)
export(facets_positioning_guide)
export(facets_term_crosswalk)
export(facets_visual_contract)
export(fair_average_table)
export(fit_measures_table)
export(fit_mfrm)
export(gpcm_capability_matrix)
export(gpcm_mml_quadrature_sensitivity)
export(gpcm_runtime_guard_coverage)
export(gpcm_score_side_contract)
export(import_erm_fit)
export(import_facets_fit_table)
export(import_mirt_fit)
export(import_tam_fit)
export(interaction_effect_table)
export(interrater_agreement_table)
export(launch_mfrmr_viewer)
export(list_mfrmr_data)
export(load_mfrmr_data)
export(make_anchor_table)
export(measurable_summary_table)
export(mfrmRFacets)
export(mfrm_d_study)
export(mfrm_generalizability)
export(mfrm_misfit_thresholds)
export(mfrm_network_analysis)
export(mfrm_report)
export(mfrm_results)
export(mfrm_results_interactive)
export(mfrm_threshold_profiles)
export(mfrmr_interval_guide)
export(mfrmr_output_guide)
export(normalize_conquest_overlap_exports)
export(normalize_conquest_overlap_files)
export(normalize_conquest_overlap_tables)
export(plot_anchor_drift)
export(plot_apa_figure_one)
export(plot_bias_interaction)
export(plot_bubble)
export(plot_data)
export(plot_data_components)
export(plot_dif_heatmap)
export(plot_dif_summary)
export(plot_displacement)
export(plot_facet_equivalence)
export(plot_facet_quality_dashboard)
export(plot_facets_chisq)
export(plot_fair_average)
export(plot_guttman_scalogram)
export(plot_information)
export(plot_interrater_agreement)
export(plot_local_dependence_heatmap)
export(plot_marginal_fit)
export(plot_marginal_pairwise)
export(plot_person_fit)
export(plot_qc_dashboard)
export(plot_qc_pipeline)
export(plot_rater_agreement_heatmap)
export(plot_rater_severity_profile)
export(plot_rater_trajectory)
export(plot_reliability_snapshot)
export(plot_residual_matrix)
export(plot_residual_pca)
export(plot_residual_qq)
export(plot_response_time_review)
export(plot_shrinkage_funnel)
export(plot_threshold_ladder)
export(plot_unexpected)
export(plot_wright_unified)
export(precision_review)
export(precision_review_report)
export(predict_mfrm_population)
export(predict_mfrm_units)
export(q3_statistic)
export(rater_halo_network_analysis)
export(rater_network_analysis)
export(rating_scale_table)
export(read_facets_fit_table)
export(recode_missing_codes)
export(recommend_mfrm_design)
export(reference_case_benchmark)
export(reference_case_review)
export(reporting_checklist)
export(response_time_review)
export(review_conquest_overlap)
export(review_mfrm_anchors)
export(run_mfrm_facets)
export(run_qc_pipeline)
export(sample_mfrm_plausible_values)
export(shrinkage_report)
export(simulate_mfrm_data)
export(specifications_report)
export(subset_connectivity_report)
export(unexpected_after_bias_table)
export(unexpected_response_table)
export(visual_reporting_template)
export(write_mfrm_residual_file)
export(write_mfrm_subset_file)
importFrom(dplyr,
  across,
  all_of,
  any_of,
  arrange,
  bind_cols,
  bind_rows,
  case_when,
  coalesce,
  count,
  desc,
  distinct,
  everything,
  filter,
  group_by,
  if_all,
  inner_join,
  lag,
  last,
  left_join,
  mutate,
  n,
  n_distinct,
  pull,
  rename,
  row_number,
  rowwise,
  select,
  slice,
  slice_head,
  slice_tail,
  summarise,
  summarize,
  transmute,
  ungroup
)
importFrom(lifecycle,deprecate_soft)
importFrom(psych,
  cor.smooth,
  describe,
  principal
)
importFrom(purrr,
  map,
  map_dbl,
  map_dfr,
  pmap_chr
)
importFrom(rlang,.data)
importFrom(stats,
  na.omit,
  optim,
  p.adjust,
  pchisq,
  pt,
  rnorm,
  sd,
  setNames,
  uniroot
)
importFrom(stringr,
  regex,
  str_count,
  str_detect,
  str_extract,
  str_pad,
  str_replace,
  str_replace_all,
  str_trunc
)
importFrom(tibble,
  as_tibble,
  column_to_rownames,
  tibble
)
importFrom(tidyr,
  drop_na,
  expand_grid,
  pivot_wider,
  replace_na,
  unite
)
importFrom(utils,
  combn,
  data,
  globalVariables,
  head,
  modifyList,
  packageVersion,
  read.csv,
  tail,
  write.csv
)
useDynLib(mfrmr, .registration = TRUE)
