# Generated by cpp11: do not edit by hand

mfrm_cpp_compute_p_geq <- function(probs) {
  .Call(`_mfrmr_mfrm_cpp_compute_p_geq`, probs)
}

mfrm_cpp_rsm_logprob_bundle <- function(score_k, person_int, base_eta, person_nodes, step_cum, weight, include_probs) {
  .Call(`_mfrmr_mfrm_cpp_rsm_logprob_bundle`, score_k, person_int, base_eta, person_nodes, step_cum, weight, include_probs)
}

mfrm_cpp_pcm_logprob_bundle <- function(score_k, person_int, base_eta, person_nodes, step_cum_obs, weight, include_probs) {
  .Call(`_mfrmr_mfrm_cpp_pcm_logprob_bundle`, score_k, person_int, base_eta, person_nodes, step_cum_obs, weight, include_probs)
}

mfrm_cpp_expected_category_bundle <- function(prob_list, obs_posterior, include_p_geq) {
  .Call(`_mfrmr_mfrm_cpp_expected_category_bundle`, prob_list, obs_posterior, include_p_geq)
}
