% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/api-import.R
\name{import_erm_fit}
\alias{import_erm_fit}
\title{Import an \code{eRm} fit to an mfrmr-compatible bundle}
\usage{
import_erm_fit(fit, model = c("RSM", "PCM", "GPCM"), item_facet = "Item")
}
\arguments{
\item{fit}{An object returned by \code{eRm::PCM()}, \code{eRm::RM()}, or
\code{eRm::RSM()}.}

\item{model}{Same as \code{\link[=import_mirt_fit]{import_mirt_fit()}}.}

\item{item_facet}{Name to assign to the item facet.}
}
\value{
An \code{mfrm_imported_fit} object.
}
\description{
Extracts item / person parameters from an \code{\link[eRm:PCM]{eRm::PCM()}} /
\code{\link[eRm:RM]{eRm::RM()}} fit. Same caveats as \code{\link[=import_mirt_fit]{import_mirt_fit()}}.
}
\seealso{
\code{\link[=import_mirt_fit]{import_mirt_fit()}}, \code{\link[=import_tam_fit]{import_tam_fit()}}
}
