% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/api-resampling.R
\name{draw_mfrm_resamples}
\alias{draw_mfrm_resamples}
\title{Draw observed-data MFRM resamples}
\usage{
draw_mfrm_resamples(spec, keep_data = TRUE)
}
\arguments{
\item{spec}{Output from \code{\link[=build_mfrm_resampling_spec]{build_mfrm_resampling_spec()}}.}

\item{keep_data}{Logical; if \code{TRUE}, return a list of replicate data frames.
If \code{FALSE}, return manifest tables only.}
}
\value{
An object of class \code{mfrm_resamples} with \code{samples}, \code{manifest},
\code{stratum_manifest}, and \code{preserve_manifest}.
}
\description{
Draw observed-data MFRM resamples
}
\seealso{
\code{\link[=build_mfrm_resampling_spec]{build_mfrm_resampling_spec()}}
}
