% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/api-advanced.R
\name{summary.mfrm_misfit_casebook}
\alias{summary.mfrm_misfit_casebook}
\title{Summarize a misfit-casebook object}
\usage{
\method{summary}{mfrm_misfit_casebook}(object, digits = 3, top_n = 10, ...)
}
\arguments{
\item{object}{Output from \code{\link[=build_misfit_casebook]{build_misfit_casebook()}}.}

\item{digits}{Number of digits for printed numeric values.}

\item{top_n}{Number of top case rows to keep in the compact summary.}

\item{...}{Reserved for generic compatibility.}
}
\value{
An object of class \code{summary.mfrm_misfit_casebook}.
}
\description{
Summarize a misfit-casebook object
}
\seealso{
\code{\link[=build_misfit_casebook]{build_misfit_casebook()}}
}
