% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/api-advanced.R
\name{summary.mfrm_model_choice_review}
\alias{summary.mfrm_model_choice_review}
\title{Summarize a model-choice review}
\usage{
\method{summary}{mfrm_model_choice_review}(object, digits = 3, ...)
}
\arguments{
\item{object}{Output from \code{\link[=build_model_choice_review]{build_model_choice_review()}}.}

\item{digits}{Number of digits for printed numeric values.}

\item{...}{Reserved for generic compatibility.}
}
\value{
An object of class \code{summary.mfrm_model_choice_review}.
}
\description{
Summarize a model-choice review
}
\seealso{
\code{\link[=build_model_choice_review]{build_model_choice_review()}}
}
