% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/api-response-time.R
\name{summary.mfrm_response_time_review}
\alias{summary.mfrm_response_time_review}
\title{Summarize a response-time review}
\usage{
\method{summary}{mfrm_response_time_review}(object, top_n = 10L, ...)
}
\arguments{
\item{object}{An object returned by \code{\link[=response_time_review]{response_time_review()}}.}

\item{top_n}{Number of top groups to retain in summary previews.}

\item{...}{Unused.}
}
\value{
A list of class \code{summary.mfrm_response_time_review}.
}
\description{
Summarize a response-time review
}
