% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/api-reports.R
\name{summary.mfrm_peer_review_design_review}
\alias{summary.mfrm_peer_review_design_review}
\title{Summarize a peer-review design review}
\usage{
\method{summary}{mfrm_peer_review_design_review}(object, digits = 3, top_n = 10, ...)
}
\arguments{
\item{object}{Output from \code{\link[=build_peer_review_design_review]{build_peer_review_design_review()}}.}

\item{digits}{Number of digits for printed numeric values.}

\item{top_n}{Number of rows to keep in compact follow-up tables.}

\item{...}{Reserved for generic compatibility.}
}
\value{
An object of class \code{summary.mfrm_peer_review_design_review}.
}
\description{
Summarize a peer-review design review
}
\seealso{
\code{\link[=build_peer_review_design_review]{build_peer_review_design_review()}}
}
