% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/api-reports.R
\name{summary.mfrm_network_review}
\alias{summary.mfrm_network_review}
\title{Summarize an MFRM network review}
\usage{
\method{summary}{mfrm_network_review}(object, digits = 3, top_n = 10, ...)
}
\arguments{
\item{object}{Output from \code{\link[=build_mfrm_network_review]{build_mfrm_network_review()}}.}

\item{digits}{Number of digits for printed numeric values.}

\item{top_n}{Number of central/cut/bridge rows to keep in the compact
summary.}

\item{...}{Reserved for generic compatibility.}
}
\value{
An object of class \code{summary.mfrm_network_review}.
}
\description{
Summarize an MFRM network review
}
\seealso{
\code{\link[=build_mfrm_network_review]{build_mfrm_network_review()}}
}
