% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/api-prediction.R
\name{summary.mfrm_plausible_values}
\alias{summary.mfrm_plausible_values}
\title{Summarize approximate plausible values from posterior scoring}
\usage{
\method{summary}{mfrm_plausible_values}(object, digits = 3, ...)
}
\arguments{
\item{object}{Output from \code{\link[=sample_mfrm_plausible_values]{sample_mfrm_plausible_values()}}.}

\item{digits}{Number of digits used in numeric summaries.}

\item{...}{Reserved for generic compatibility.}
}
\value{
An object of class \code{summary.mfrm_plausible_values} with:
\itemize{
\item \code{draw_summary}: empirical summaries of the sampled values by person
\item \code{estimates}: companion posterior EAP summaries
\item \code{row_review}: row-preparation review
\item \code{population_review}: optional person-level omission review for
latent-regression scoring
\item \code{settings}: scoring settings
\item \code{notes}: interpretation notes
}
}
\description{
Summarize approximate plausible values from posterior scoring
}
\examples{
toy <- load_mfrmr_data("example_core")
keep_people <- unique(toy$Person)[1:18]
toy_fit <- suppressWarnings(
  fit_mfrm(
    toy[toy$Person \%in\% keep_people, , drop = FALSE],
    "Person", c("Rater", "Criterion"), "Score",
    method = "MML",
    quad_points = 5,
    maxit = 30
  )
)
new_units <- data.frame(
  Person = c("NEW01", "NEW01"),
  Rater = unique(toy$Rater)[1],
  Criterion = unique(toy$Criterion)[1:2],
  Score = c(2, 3)
)
pv <- sample_mfrm_plausible_values(toy_fit, new_units, n_draws = 3, seed = 1)
summary(pv)
}
\seealso{
\code{\link[=sample_mfrm_plausible_values]{sample_mfrm_plausible_values()}}
}
